{-# LANGUAGE OverloadedStrings #-}
module Main where

import Data.Text(Text, pack, unpack)
import Data.Maybe(fromMaybe, isJust, fromJust)
import Data.String
import Data.List(intercalate)
import Data.ByteString(ByteString) -- hiding (pack, unpack, intercalate)

import System.IO.Error(isUserError)
import Control.Exception(catchJust)
import Control.Monad(join)

import Network.Wai
import Network.HTTP.Types (status200)
import Network.Wai.Handler.Warp
import Network.HTTP.Types.URI
import Network.HTTP.Types.Method

import qualified Text.Blaze.Html5 as H -- hiding (main,map)
import Text.Blaze((!))
import Text.Blaze.Html5.Attributes as A
import Text.Blaze.Html.Renderer.Utf8

import qualified GridMaze

mainPage :: H.Markup -> H.Html
mainPage mazeOrNot =
  H.docTypeHtml $ do
    H.head $ do
      H.title "Amazing Mazes"
    H.body $ do
      H.h1 "Amazing Mazes"
      H.br
      mazeOrNot
      H.br
      H.form  $ do
        H.h2 "Maze Parameters"
        H.text "Width: "
        H.input ! A.type_ "text" ! A.value "30" ! A.name "width"
        H.br ! A.style "margin-bottom: 1em;"
        H.text "Height: "
        H.input ! A.type_ "text" ! A.value "40" ! A.name "height"
        H.br ! A.style "margin-bottom: 1em;"
        H.text "Cellsize: "
        H.input ! A.type_ "text" ! A.value "5" ! A.name "cellsize"
        H.br ! A.style "margin-bottom: 1em;"
        H.input ! A.type_ "submit" ! A.value "Create new maze!"

noMaze :: H.Html
noMaze = mainPage $ H.span "You have not created a maze yet!"

errorPage :: H.Html
errorPage = mainPage $ do H.b $ H.span "ERROR: "; H.text "Parameters (width, height, cellsize) must be valid integers."

readMaybe :: Read a=> String-> IO (Maybe a)
readMaybe s = catchJust (\e-> if isUserError e then Just () else Nothing)
                        (readIO s >>= return . Just) (const $ return Nothing)

newMaze :: Int-> Int-> Int -> IO H.Html
newMaze = undefined

page :: QueryText -> IO H.Html
page qt
  | checkParams = do
     w <- readParam "width"
     h <- readParam "height"
     s <- readParam "cellsize"
     if isJust w && isJust h && isJust s then
       newMaze (fromJust w) (fromJust h) (fromJust s)
     else
       return $ errorPage 
  | otherwise = return $ noMaze
    where 
      checkParams = all (\p-> isJust (join (lookup (pack p) qt)))
                        ["width", "height", "cellsize"]
      readParam p = readMaybe (unpack $ fromJust $ join $ lookup (pack p) qt)

-- The Maze application
mazeApp :: Application
mazeApp req respond = do
      p <- page (queryToQueryText $ queryString req)
      respond $ responseBuilder status200 [] $ renderHtmlBuilder p

-- Run on port 8000
main = do
  putStrLn "Server running, connect to http://localhost:8000"
  run 8000 $ mazeApp

